//@version=3

// BEGIN UNCOMMENT FOR ALERTS
study(title="zigzag%", shorttitle="zigzag%", overlay=true)
strategyEntry(RUNNING, AS, STUDY) =>   RUNNING=='false' ? false : false 
strategyClose(ALERTS, NOT_STRATEGY) => ALERTS== 'false' ? false : false
isStrategy = false, isStudy = true
//

ZZPercent = input(1.5, title="Minimum % Change", type=float)

trend = 0
trend := na(trend[1]) ? 1 : trend[1]
LL = 0.0
LL := na(LL[1]) ? low : LL[1]
HH = 0.0
HH := na(HH[1]) ? high : HH[1]

PivotChg = ZZPercent * 0.01
zigzag = na

if (trend > 0)       // trend is up, look for new swing low
    if (high >= HH)  // new higher high detected
	    HH := high
	else
	    if (low < HH * (1 - PivotChg))
	    	zigzag := high[1]
		    trend := -1
		    LL := low
else                 // trend is down, look for new swing high
    if (low <= LL)   // new lower low detected
	    LL := low
	else
	    if (high > LL * (1 + PivotChg))
			zigzag := low[1]
		    trend := 1
			HH := high

plot(zigzag, linewidth=2, offset=0)

f=crossunder(trend,0)
q=crossover(trend,0)



useRenko   = input(false, title="This a RENKO Chart?")
// WARNING: Using Heikin-Ashi may result in inaccurate backtest results
useHeikin  = input(false, title="Use Heikin-Ashi Candles? (BT WILL BE UNRELIABLE)")
// Use Alternate Anchor TF for MAs 
anchor = input(0,minval=0,maxval=1440,title="Alternate TimeFrame Multiplier (0=none)")
// If have anchor specified, calculate the base multiplier.
//mult  = isintraday ? anchor==0 or interval<=0 or interval>=anchor or anchor>1440? 1 : round(anchor/interval) : 1
//mult := isdwm?  1 : mult  // Only available Daily or less
mult = anchor>0 ? anchor : 1 
open_  = useHeikin ? security(heikinashi(tickerid), period, open) : security(tickerid, period, open)
close_ = useHeikin ? security(heikinashi(tickerid), period, close) : security(tickerid, period, close)
high_  = useRenko? max(close_,open_) : useHeikin ? security(heikinashi(tickerid), period, high) : security(tickerid, period, high)
low_   = useRenko? min(close_,open_) : useHeikin ? security(heikinashi(tickerid), period, low) :  security(tickerid, period, low)
uPrice = input(title="Price", defval="close", options=["close", "high", "low"])
src= uPrice=="close" ? close_ : uPrice=="high" ? high_ : uPrice=="low" ? low_ : na
// This is superced by the design of specifying Open and Close setting in strategy
// fastExit  = input(false,title="Use Opposite Trade as a Close Signal")
fastExit = false
isLong = input(true,title="Allow Longs")
isShort = input(true,title="Allow Shorts")
flipTrade = input(false,title="Flip/Invert Strategy")
clrBars   = input(false,title="Colour Candles to Trade Order state")

// CONSTANTS
AQUA = #00FFFFFF
BLUE = #0000FFFF
RED  = #FF0000FF
LIME = #00FF00FF
GRAY = #808080FF
DARKRED   = #8B0000FF
DARKGREEN = #006400FF

//////////////////////////////////////////////////////////////////////////////////////////
// BEGIN ACTUAL STRATEGY

// Strategy specific inputs
quick = input(14, title="Quick Period", step=1, minval=1) 
slow = input(28, title="Slow Period", step=1, minval=1) 

// UPDATE WITH ACTUAL STRATEGY HERE

openLong   = zigzag and q 
closeLong  =  zigzag and f 

openShort  = closeLong
closeShort = openLong

// END ACTUAL STRATEGY
//////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////
//* Period selector for backtesting and optionally alerts

// A switch to control background coloring of the test period
testPeriodforAlerts = input(title="Limit Alerts to Test Period", type=bool, defval=false)
testPeriodBackground = input(title="Color Test Period?", type=bool, defval=false)

// Autoset Backtest Period
autolen = input(title="Auto set backtest Start", defval=false)
custom_candle_start = 0
timecondition_hourly = n > round(76.5633 * pow((interval / 60), 2) - 1745.19 * (interval / 60) + 10649)
timecondition_daily = n > round(1000 / interval) // Only for bitcoin, need a separate quadratic fit for other securities
autolen_timePeriod = autolen ? (interval >= 60 and interval <= 720 and timecondition_hourly) or (interval < 60 and timecondition_daily) : false
    
// Or set a specifc period
testStartMonth = input(1, "Backtest Start Month",minval=1,maxval=12)
testStartYear = input(2017, "Backtest Start Year",minval=1980)
// testStartDay = input(1, "Backtest Start Day",minval=1,maxval=31)
testStartDay = 1
testPeriodStart = timestamp(testStartYear,testStartMonth,testStartDay,0,0)

testStopMonth = input(title="Backtest Stop Month", type=integer, defval=12, step=1, minval=1, maxval=12)
testStopYear = input(title="Backtest Stop Year", type=integer, defval=2018, step=1, minval=2013, maxval=2020)
//testStopDay = input(title="Backtest Stop Day", type=integer, defval=31, step=1, minval=1, maxval=31)
testStopDay=31
testPeriodStop = timestamp(testStopYear,testStopMonth,testStopDay,0,0)

// See if we are in the the time period (or we are not using it)
testPeriod() =>
    // Are we running as a strategy or as a Study with testPeriod restriction?
    isStrategy or testPeriodforAlerts ?
      autolen ? autolen_timePeriod :
      time >= testPeriodStart and time <= testPeriodStop ? true : false
      : true // Not a strategy or not using testPeriod so it is "always" the period
    
// Color the period (or don't)
testPeriodBackgroundColor = testPeriodBackground and testPeriod() ? #000055 : na
bgcolor(testPeriodBackgroundColor, transp=90)

//////////////////////////////////////////////////////////////////////////////////////////
// Generate Buy Sell signals, 

buy = 0, buy  := openLong  and testPeriod() ? nz(buy[1])>0?buy[1]+1  :1 : 0
sell= 0, sell := openShort and testPeriod() ? nz(sell[1])>0?sell[1]+1 :1 : 0


//////////////////////////
//* Trade State Engine *//
//////////////////////////

// Keep track of current trade state
longClose = false, longClose := nz(longClose[1],false)
shortClose = false, shortClose := nz(shortClose[1],false)
tradeState = 0, tradeState := nz(tradeState[1])
tradeState := tradeState==0 ?   buy==1 and (barstate.isconfirmed or barstate.ishistory) and isLong and not longClose and not shortClose? 1 :
                               sell==1  and (barstate.isconfirmed or barstate.ishistory) and isShort and not longClose and not shortClose? -1 : 
                          tradeState : tradeState

///////////////////////////////////////////////////////
//Handle Entry Conditions, when state changes direction.

longCondition  = false
shortCondition = false
longCondition  := change(tradeState) and tradeState==1
shortCondition := change(tradeState) and tradeState==-1
if flipTrade
    temp = longCondition
    longCondition  := shortCondition
    shortCondition := temp
//end if

// Exit on Sell signal
longExitC  = closeLong  and testPeriod() ? 1 : 0
shortExitC = closeShort and testPeriod() ? 1 : 0

// Exit condition for no SL.
longExit = change(longExitC) and longExitC==1 and tradeState==1
shortExit = change(shortExitC) and shortExitC==1 and tradeState==-1

// -- debugs
//plotchar(tradeState,"tradeState at Event",location=location.bottom, color=na)
//plotchar(longCondition, title="longCondition",color=na)
//plotchar(shortCondition, title="shortCondition",color=na)
//plotchar(tradeState, title="tradeState",color=na)
// -- /debugs

/////////////////////////////////////
//======[ Deal Entry Prices ]======//
/////////////////////////////////////

last_open_longCondition = na
last_open_shortCondition = na
last_open_longCondition := longCondition ? close : nz(last_open_longCondition[1])
last_open_shortCondition := shortCondition ? close : nz(last_open_shortCondition[1])

//////////////////////////////////
//======[ Position State ]======//
//////////////////////////////////

in_longCondition  = tradeState == 1 
in_shortCondition = tradeState == -1

/////////////////////////////////
//======[ Trailing Stop ]======//
/////////////////////////////////

isTS = input(true, "Trailing Stop")
ts = input(2.0, "Trailing Stop (%)", minval=0,step=0.1, type=float) /100

last_high = na
last_low = na
last_high_short = na
last_low_long = na
last_high := not in_longCondition ? na : in_longCondition and (na(last_high[1]) or high_ > nz(last_high[1])) ? high_ : nz(last_high[1])
last_high_short := not in_shortCondition ? na : in_shortCondition and (na(last_high[1]) or high_ > nz(last_high[1])) ? high_ : nz(last_high[1])
last_low := not in_shortCondition ? na : in_shortCondition and (na(last_low[1]) or low_ < nz(last_low[1])) ? low_ : nz(last_low[1])
last_low_long := not in_longCondition ? na : in_longCondition and (na(last_low[1]) or low_ < nz(last_low[1])) ? low_ : nz(last_low[1])

long_ts =  isTS and in_longCondition and not na(last_high) and (low_ <= last_high - last_high * ts) //and (last_high >= last_open_longCondition + last_open_longCondition * tsi)
short_ts = isTS and in_shortCondition and not na(last_low) and (high_ >= last_low + last_low * ts) //and (last_low <= last_open_shortCondition - last_open_shortCondition * tsi)


///////////////////////////////
//======[ Take Profit ]======//
///////////////////////////////

isTP = input(true, "Take Profit")
tp = input(2.0, "Take Profit (%)",minval=0,step=0.1,type=float) / 100
ttp = input(0.5, "Trailing Profit (%)",minval=0,step=0.1,type=float) / 100
ttp := ttp>tp ? tp : ttp

long_tp =  isTP and in_longCondition  and (last_high >= last_open_longCondition + last_open_longCondition * tp)   and (low_ <= last_high - last_high * ttp)
short_tp = isTP and in_shortCondition and (last_low <= last_open_shortCondition - last_open_shortCondition * tp) and (high_ >= last_low + last_low * ttp)

/////////////////////////////
//======[ Stop Loss ]======//
/////////////////////////////

isSL = input(true, "Stop Loss")
sl = input(2.0, "Stop Loss (%)", minval=0,step=0.1, type=float) / 100
long_sl =  isSL and in_longCondition and (low_ <= last_open_longCondition - last_open_longCondition * sl)
short_sl = isSL and in_shortCondition and (high_ >= last_open_shortCondition + last_open_shortCondition * sl)

////////////////////////////////////
//======[ Stop on Opposite ]======//
////////////////////////////////////
// fastExit is superced by the design of specifying Open and Close setting in strategy
long_sos = (fastExit or (not isTS and not isSL)) and longExit and in_longCondition
short_sos = (fastExit or (not isTS and not isSL)) and shortExit and in_shortCondition 


/////////////////////////////////
//======[ Close Signals ]======//
/////////////////////////////////

// Create a single close for all the different closing conditions, all conditions here are non-repainting
longClose :=  isLong and (long_tp or long_sl or long_ts or long_sos) and not longCondition
shortClose := isShort and (short_tp or short_sl or short_ts or short_sos) and not shortCondition

///////////////////////////////
//======[ Plot Colors ]======//
///////////////////////////////

longCloseCol = na
shortCloseCol = na
longCloseCol := long_tp ? green : long_sl ? maroon : long_ts ? purple : long_sos ? orange :longCloseCol[1]
shortCloseCol := short_tp ? green : short_sl ? maroon : short_ts ? purple : short_sos ? orange : shortCloseCol[1]
//
tpColor = isTP and in_longCondition ? lime : isTP and in_shortCondition ? lime : na
slColor = isSL and in_longCondition ? red : isSL and in_shortCondition ? red : na


//////////////////////////////////
//======[ Strategy Plots ]======//
//////////////////////////////////

plot(isTS and in_longCondition?
     last_high - last_high * ts : na, "Long Trailing", fuchsia, style=2, linewidth=2,offset=1)
plot(isTP and in_longCondition and last_high < last_open_longCondition + last_open_longCondition * tp ? 
     last_open_longCondition + last_open_longCondition * tp : na, "Long TP Active", tpColor, style=3,join=false, linewidth=2,offset=1)
plot(isTP and in_longCondition and last_high >= last_open_longCondition +  last_open_longCondition * tp ? 
     last_high - last_high * ttp : na, "Long Trailing", black, style=2, linewidth=2,offset=1)
plot(isSL and in_longCondition and last_low_long > last_open_longCondition - last_open_longCondition * sl ? 
     last_open_longCondition - last_open_longCondition * sl : na, "Long SL", slColor, style=3,join=false, linewidth=2,offset=1)
//
plot(isTS and in_shortCondition?
     last_low + last_low * ts : na, "Short Trailing", fuchsia, style=2, linewidth=2,offset=1)
plot(isTP and in_shortCondition and last_low > last_open_shortCondition - last_open_shortCondition * tp ? 
     last_open_shortCondition - last_open_shortCondition * tp : na, "Short TP Active", tpColor, style=3,join=false, linewidth=2,offset=1)
plot(isTP and in_shortCondition and last_low <= last_open_shortCondition -  last_open_shortCondition * tp ? 
     last_low + last_low * ttp : na, "Short Trailing", black, style=2, linewidth=2,offset=1)
plot(isSL and in_shortCondition and last_high_short < last_open_shortCondition + last_open_shortCondition * sl ? 
     last_open_shortCondition + last_open_shortCondition * sl : na, "Short SL", slColor, style=3,join=false, linewidth=2,offset=1)
//
bclr = not clrBars ? na : tradeState==0 ? GRAY : 
                          in_longCondition ? close<last_open_longCondition? DARKGREEN : LIME :
                          in_shortCondition ? close>last_open_shortCondition? DARKRED : RED : GRAY
barcolor(bclr,title="Trade State Bar Colouring")


///////////////////////////////
//======[ Alert Plots ]======//
///////////////////////////////

plotshape(longCondition and isStudy?close:na, title="Long", color=green, textcolor=green, transp=0, 
          style=shape.triangleup, location=location.belowbar, size=size.small,text="LONG",offset=0)
plotshape(longClose and isStudy?close:na, title="Long Close", color=longCloseCol, textcolor=white, transp=0, 
          style=shape.labeldown, location=location.abovebar, size=size.small,text="Long\nClose",offset=0)

plotshape(shortCondition and isStudy?close:na, title="Short", color=red, textcolor=red, transp=0, 
          style=shape.triangledown, location=location.abovebar, size=size.small,text="SHORT",offset=0)
plotshape(shortClose and isStudy?close:na, title="Short Close", color=shortCloseCol, textcolor=white, transp=0, 
          style=shape.labelup, location=location.belowbar, size=size.small,text="Short\nClose",offset=0)

// Autoview alert syntax - This assumes you are trading coins BUY and SELL on Binance Exchange
// WARNING*** Only use Autoview to automate a strategy after you've sufficiently backtested and forward tested the strategy.
// You can learn more about the syntax here:
//      http://autoview.with.pink/#syntax and you can watch this video here: https://www.youtube.com/watch?v=epN5Tjinuxw

// For the opens you will want to trigger BUY orders on LONGS (eg ETHBTC) with alert option "Once Per Bar Close"
// and SELL orders on SHORTS (eg BTCUSDT)
//      b=buy q=0.001 e=binance s=ethbtc t=market  ( LONG )
// or   b=sell q=0.001 e=binance s=btcusdt t=market ( SHORT )
alertcondition(longCondition and isStudy, "Open Long", "LONG")
alertcondition(shortCondition and isStudy, "Open Short", "SHORT")

// For the closes you will want to trigger these alerts on condition with alert option "Once Per Bar"
// (NOTE: with Renko you can only use "Once Per Bar Close" option)
//      b=sell q=99% e=binance s=ethbtc t=market  ( CLOSE LONGS )
// or   b=buy q=99% e=binance s=btcusdt t=market  ( CLOSE SHORTS )
// This gets it as it happens and typically results in a better exit live than in the backtest. 
// It works really well for counteracting some market slippage
alertcondition(longClose and isStudy, "Close Longs", "CLOSE LONGS")
alertcondition(shortClose and isStudy, "Close Shorts", "CLOSE SHORTS")

////////////////////////////////////////////
//======[ Strategy Entry and Exits ]======//
////////////////////////////////////////////
if testPeriod() and isLong and isStrategy
    strategyEntry("Long", 1, longCondition)
    strategyClose("Long", longClose)

if testPeriod() and isShort and isStrategy
    strategyEntry("Short", 0, shortCondition)
    strategyClose("Short", shortClose)
    
// --- Debugs
//plotchar(longExit,title="longExit",location=location.bottom,color=na)
//plotchar(longCondition,title="longCondition",location=location.bottom,color=na)
//plotchar(in_longCondition,title="in_longCondition",location=location.bottom,color=na)
//plotchar(longClose,title="longClose",location=location.bottom,color=na,color=na)
//plotchar(buy,title="buy",location=location.bottom,color=na)
// --- /Debugs

///////////////////////////////////
//======[ Reset Variables ]======//
///////////////////////////////////

if longClose or not in_longCondition
    last_high := na
    last_high_short := na
    
if shortClose or not in_shortCondition
    last_low := na
    last_low_long := na
    
if longClose or shortClose
    tradeState := 0
    in_longCondition := false
    in_shortCondition := false

    
plotchar(tradeState,"tradeState at EOF",location=location.bottom, color=na)
// EOF